@@ -3,10 +3,11 @@ from __future__ import print_function
 
 import os
 
+from clu.abstract import AppName, Cloneable, ReprWrapper
 from clu.constants.consts import PROJECT_NAME, NoDefault
-from clu.config.abc import NamespacedMutableMapping, NAMESPACE_SEP
-from clu.config.base import AppName
-from clu.exporting import Exporter
+from clu.config.abc import NAMESPACE_SEP, NamespacedMutableMapping
+from clu.predicates import tuplize
+from clu.exporting import Slotted, Exporter
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
@@ -14,7 +15,10 @@ export = exporter.decorator()
 PREFIX_SEP = '_'
 
 @export
-class EnvBase(NamespacedMutableMapping, AppName):
+class EnvBase(NamespacedMutableMapping, AppName,
+                                        Cloneable,
+                                        ReprWrapper,
+                                        metaclass=Slotted):
     
     """ The base class for “clu.config.env.Env”. Override this class in
         your own project for access to an interface to the environment
@@ -26,6 +30,7 @@ class EnvBase(NamespacedMutableMapping, AppName):
         class keyword and furnishes a read-only descriptor based on that
         keywords’ value.
     """
+    __slots__ = tuplize('environment')
     
     @classmethod
     def prefix(cls, namespace=None):
@@ -148,7 +153,14 @@ class EnvBase(NamespacedMutableMapping, AppName):
         prefixes = frozenset(key.lower().split(PREFIX_SEP, 1)[0] for key in envkeys if PREFIX_SEP in key)
         return tuple(sorted(prefixes))
     
-    def clone(self):
+    def inner_repr(self):
+        """ Return some readable meta-information about this instance """
+        prefix = type(self).prefix(namespace=None)
+        namespaces = len(self.namespaces())
+        keys = len(list(self.keys()))
+        return f"[prefix=“{prefix}*”, namespaces={namespaces}, keys={keys}]"
+    
+    def clone(self, *args, **kwargs):
         """ Return a cloned copy of this NamespacedMutableMapping environment
             interface.
         """
