@@ -2,20 +2,164 @@
 from __future__ import print_function
 
 import abc
+import os
+
+abstract = abc.abstractmethod
 
 from clu.constants.consts import ENCODING, PYTHON_VERSION
 from clu.config.base import Flat, Nested
 from clu.config.fieldtypes import FieldBase
 from clu.fs.misc import stringify
-from clu.predicates import haspyattr, getpyattr, stattr, pyattrs, iscontainer, no_op
-from clu.typology import ismapping
-from clu.exporting import Exporter
+from clu.predicates import (haspyattr, getpyattr,
+                            stattr, pyattrs,
+                            always, no_op,
+                            iscontainer, slots_for)
+from clu.typology import differentlength, ismapping, isstring
+from clu.exporting import Slotted, Exporter
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
 NEED_NAME = PYTHON_VERSION < 3.6
 
+class Nestifier(abc.ABC):
+    
+    __slots__ = tuple()
+    
+    @abstract
+    def namespaced_fields(self):
+        """ Return a NamespacedMutableMapping of fields """
+        ...
+    
+    def nestify(self, stringify=False, **kwargs):
+        """ Return an instance of “clu.config.base.Nested” – a concrete
+            NamespacedMutableMapping subclass – containing all of the data from
+            the Schema instance – including any embedded sub-Schema data.
+        """
+        converter = stringify and str or no_op
+        out = Nested()
+        for key, value in self.namespaced_fields().items():
+            keywords = dict(stringify=stringify,
+                            instance=kwargs.get('instance', self))
+            if haspyattr(value, 'json'):
+                out[key] = value.__json__(**keywords)
+            elif iscontainer(value):
+                out[key] = tuple(v.__json__(**keywords) if haspyattr(v, 'json') else v for v in value)
+            elif ismapping(value):
+                out[key] = dict((k, v.__json__(**keywords) if haspyattr(v, 'json') else k, v) \
+                             for k, v in value.items())
+            else:
+                out[key] = (value is not None) and converter(value) or value
+        return out
+    
+    def __json__(self, **kwargs):
+        """ Return a nested set of dicts, suitable for serializing as JSON
+            (and other similar formats).
+        """
+        return self.nestify(instance=self, **kwargs).tree
+
+class Namespace(FieldBase, Nestifier, metaclass=Slotted):
+    
+    __slots__ = ('namespaced_dict',
+                 'namespace',
+                 'initialized')
+    
+    slots = tuple() # This will be filled in below
+    
+    def __new__(cls, *args, **kwargs):
+        try:
+            instance = super(Namespace, cls).__new__(cls, *args, **kwargs)
+        except TypeError:
+            instance = super(Namespace, cls).__new__(cls)
+        instance.namespaced_dict = Flat()
+        instance.default = Flat()
+        instance.namespace = None
+        instance.allow_none = False
+        instance.validator = always
+        instance.extractor = no_op
+        instance.initialized = False
+        return instance
+    
+    def __init__(self, namespaced_dict, namespace=None):
+        if not namespaced_dict:
+            raise ValueError("A truthy namespaced dictionary is required")
+        if not namespace:
+            raise ValueError("A truthy namespace declaration is required")
+        if not isstring(namespace):
+            raise ValueError("A string namespace declaration is required")
+        self.namespaced_dict.update(
+             namespaced_dict.items(namespace))
+        self.namespace = namespace
+        self.initialized = True
+    
+    def __getattr__(self, key):
+        try:
+            return self.namespaced_fields().get(key, namespace=self.namespace)
+        except KeyError:
+            return object.__getattribute__(self, key)
+    
+    def __setattr__(self, key, value):
+        print(f"SETATTR: {key} : {value}")
+        try:
+            if super(Namespace, self).__getattribute__('initialized') == True:
+                print("INITIALIZED")
+                # if self.pack_ns(key, namespace=self.namespace) in self.namespaced_fields():
+                if key not in self.slots:
+                    print(f"NAMESPACED FIELDS SET: {key} : {value}")
+                    self.namespaced_fields().set(key, value, namespace=self.namespace)
+                    return
+        except AttributeError:
+            pass
+        object.__setattr__(self, key, value)
+    
+    def get_default(self):
+        return self
+    
+    def namespaced_fields(self):
+        return self.default
+    
+    @property
+    def name(self):
+        return self.namespace
+    
+    @name.setter
+    def name(self, value):
+        self.namespace = str(value)
+    
+    def update(self, instance):
+        """ Update the live-data fields from a Nestifier instance """
+        if instance is None:
+            return
+        if differentlength(tuple(instance.namespaced_fields().keys(namespace=self.namespace)),
+                                     self.namespaced_fields()):
+            self.namespaced_fields().update(
+        instance.namespaced_fields().items(
+                 namespace=self.namespace))
+    
+    def __set_name__(self, cls, name):
+        self.name = name
+    
+    def __get__(self, instance, cls=None):
+        self.update(instance)
+        return self
+    
+    def __set__(self, instance, value):
+        self.update(instance)
+        return value
+    
+    def __json__(self, **kwargs):
+        return self.namespaced_fields().nestify().tree
+    
+    def __str__(self):
+        return self.namespace
+    
+    def __repr__(self):
+        # return stringify(self, self.namespaced_fields().keys())
+        return repr(self.namespaced_fields())
+
+# Set the “slots” class attribute once and only once:
+Namespace.slots = slots_for(Namespace)
+
 class MetaSchema(abc.ABCMeta):
     
     """ The MetaSchema metaclass is used by the Schema class – q.v.
@@ -59,6 +203,13 @@ class MetaSchema(abc.ABCMeta):
                     field_index.set(attribute, value,
                                     namespace=value.namespace)
         
+        for namespace in field_index.namespaces():
+            nsfield = Namespace(field_index, namespace=namespace)
+            if NEED_NAME:
+                nsfield.__set_name__(None, namespace)
+            attributes[namespace] = nsfield
+            field_names[namespace] = nsfield
+        
         # Add both the field-index and the field-names mappings
         # to the class dictionary for the new type:
         attributes['__field_index__'] = field_index
@@ -71,7 +222,7 @@ class MetaSchema(abc.ABCMeta):
                                                          **kwargs)
 
 @export
-class Schema(abc.ABC, metaclass=MetaSchema):
+class Schema(Nestifier, metaclass=MetaSchema):
     
     """ The Schema class is the root class for all CLU configuration schemas.
         
@@ -157,33 +308,21 @@ class Schema(abc.ABC, metaclass=MetaSchema):
             if key in field_names:
                 setattr(instance, key, value)
         
+        for namespace in instance.__fields__.namespaces():
+            if namespace in field_names:
+                setattr(instance, namespace, field_names[namespace])
+        
         # Return the new instance:
         return instance
     
+    def namespaced_fields(self):
+        return self.__fields__
+    
     def namespaces(self):
         """ Return a sorted tuple of all of the namespaces that have been
             defined on this Schema.
         """
-        return self.__fields__.namespaces()
-    
-    def nestify(self, stringify=False):
-        """ Return an instance of “clu.config.base.Nested” – a concrete
-            NamespacedMutableMapping subclass – containing all of the data from
-            the Schema instance – including any embedded sub-Schema data.
-        """
-        converter = stringify and str or no_op
-        out = Nested()
-        for key, value in self.__fields__.items():
-            if haspyattr(value, 'json'):
-                out[key] = value.__json__(stringify=stringify)
-            elif iscontainer(value):
-                out[key] = tuple(v.__json__(stringify=stringify) if haspyattr(v, 'json') else v for v in value)
-            elif ismapping(value):
-                out[key] = dict((k, v.__json__(stringify=stringify) if haspyattr(v, 'json') else k, v) \
-                             for k, v in value.items())
-            else:
-                out[key] = (value is not None) and converter(value) or value
-        return out
+        return self.namespaced_fields().namespaces()
     
     def __json__(self, **kwargs):
         """ Return a nested set of dicts, suitable for serializing as JSON
@@ -254,8 +393,8 @@ class Schema(abc.ABC, metaclass=MetaSchema):
         field_names, field_index = pyattrs(type(self), 'field_names',
                                                        'field_index')
         for field, nsfield in zip(field_names, field_index):
-            setattr(self, field, self.__fields__[nsfield])
-        for value in self.__fields__.values():
+            setattr(self, field, self.namespaced_fields()[nsfield])
+        for value in self.namespaced_fields().values():
             if hasattr(value, 'validate'):
                 value.validate()
             elif iscontainer(value):
@@ -269,9 +408,13 @@ __all__, __dir__ = exporter.all_and_dir()
 
 def test():
     from clu.config.fieldtypes import fields
-    from clu.repl.ansi import print_separator
+    from clu.config.formats import JsonFile
+    from clu.fs.filesystem import TemporaryName
+    from clu.repl.ansi import print_separator, print_ansi_centered, ANSIFormat
     from pprint import pprint
     
+    nullformat = ANSIFormat.null()
+    
     class MySchema(Schema):
         
         title = fields.String("«title»")
@@ -285,97 +428,154 @@ def test():
         
         with fields.ns('yodogg'):
             
-            yodogg = fields.String("Yo dogg,")
             iheard = fields.String("I heard")
+            youlike = fields.String("you like:")
             andalso = fields.Tuple(value=fields.String("«also»", allow_none=False))
     
-    instance = MySchema()
-    instance.validate()
+    def test_one():
+        
+        instance = MySchema()
+        instance.validate()
+        
+        print_separator(filler='•')
+        print_ansi_centered("TEST ONE", color=nullformat)
+        print_separator(filler='•')
+        print()
+        
+        print("str(instance.yodogg) =",  str(instance.yodogg))
+        print()
+        print("repr(instance.yodogg) =", repr(instance.yodogg))
+        print()
+        print("repr(instance.yodogg.slots) =", repr(instance.yodogg.slots))
+        print()
+        print("str(instance.yodogg.iheard) =",  str(instance.yodogg.iheard))
+        print()
+        print("repr(instance.yodogg.iheard) =", repr(instance.yodogg.iheard))
+        print()
+        
+        instance.yodogg = Namespace(instance.yodogg.default.clone(), 'yodogg')
+        instance.yodogg.iheard = 'I REALLY DID HEAR'
+        
+        print_separator()
+        
+        print("str(instance.yodogg.iheard) =",  str(instance.yodogg.iheard))
+        print()
+        print("repr(instance.yodogg.iheard) =", repr(instance.yodogg.iheard))
+        print()
+        
+        print_separator()
+        
+        print("» JSON:")
+        print()
+        # print(instance.to_json())
+        
+        with TemporaryName(prefix='temp-config-settings-',
+                           suffix='json',
+                           randomize=True) as tj:
+            jsonfile = JsonFile(tj.name, filename=os.path.split(tj.name)[-1])
+            jsonfile.update(instance.nestify(stringify=False))
+            json = jsonfile.dumps()
+            # pprint(jsonfile.tree)
+        
+        assert not jsonfile.exists
+        print(json)
+        print()
+        print_separator()
+        
+        # YOU CAN’T HANDLE THE “NoneType”:
+        # print("» PLIST:")
+        # print()
+        # print(instance.to_plist())
+        # print()
+        # print_separator()
+        
+        print("» TOML:")
+        print()
+        print(instance.to_toml())
+        print()
+        print_separator()
+        
+        # print("» YAML:")
+        # print()
+        # print(instance.to_yaml())
+        # print()
+        # print_separator()
+        
+        print("» __repr__(…):")
+        print()
+        print(repr(instance))
+        print()
+        print_separator()
+        
+        print("» namespaces:")
+        print()
+        pprint(instance.namespaces())
+        print()
+        # print_separator()
     
-    print_separator()
+    def test_two():
+        print_separator(filler='•')
+        print_ansi_centered("TEST TWO", color=nullformat)
+        print_separator(filler='•')
+        print()
+        
+        instance0 = MySchema(title="YO DOGG", count=666, where=-8)
+        instance0.validate()
+        
+        instance0.considerations = "Whatever man."
+        instance0.iheard = "Actually I haven’t heard."
+        instance0.andalso = ('additionally', 'we', 'put', 'some', 'strings')
+        
+        print("» JSON:")
+        print()
+        print(instance0.to_json())
+        print()
+        
+        # YOU CAN’T HANDLE THE “NoneType”:
+        # print_separator()
+        # print("» PLIST:")
+        # print()
+        # print(instance0.to_plist())
+        # print()
+        
+        print_separator()
+        print("» TOML:")
+        print()
+        print(instance0.to_toml())
+        print()
+        
+        print_separator()
+        print("» YAML:")
+        print()
+        print(instance0.to_yaml())
+        print()
+        
+        print_separator()
+        print("» __repr__(…):")
+        print()
+        print(repr(instance0))
+        print()
+        
+        print_separator()
+        print("» __str__(…):")
+        print()
+        print(str(instance0))
+        print()
+        
+        print_separator()
+        print("» __bytes__(…):")
+        print()
+        print(bytes(instance0))
+        print()
+        
+        print_separator()
+        print("» namespaces:")
+        print()
+        pprint(instance0.namespaces())
+        print()
     
-    print("» JSON:")
-    print()
-    print(instance.to_json())
-    print()
-    
-    # YOU CAN’T HANDLE THE “NoneType”:
-    # print("» PLIST:")
-    # print()
-    # print(instance.to_plist())
-    # print()
-    
-    print("» TOML:")
-    print()
-    print(instance.to_toml())
-    print()
-    
-    print("» YAML:")
-    print()
-    print(instance.to_yaml())
-    print()
-    
-    print("» __repr__(…):")
-    print()
-    print(repr(instance))
-    print()
-    
-    print("» namespaces:")
-    print()
-    pprint(instance.namespaces())
-    print()
-    
-    print_separator()
-    
-    instance0 = MySchema(title="YO DOGG", count=666, where=-8)
-    instance0.validate()
-    
-    instance0.considerations = "Whatever man."
-    instance0.iheard = "Actually I haven’t heard."
-    instance0.andalso = ('additionally', 'we', 'put', 'some', 'strings')
-    
-    print("» JSON:")
-    print()
-    print(instance0.to_json())
-    print()
-    
-    # YOU CAN’T HANDLE THE “NoneType”:
-    # print("» PLIST:")
-    # print()
-    # print(instance0.to_plist())
-    # print()
-    
-    print("» TOML:")
-    print()
-    print(instance0.to_toml())
-    print()
-    
-    print("» YAML:")
-    print()
-    print(instance0.to_yaml())
-    print()
-    
-    print("» __repr__(…):")
-    print()
-    print(repr(instance0))
-    print()
-    
-    print("» __str__(…):")
-    print()
-    print(str(instance0))
-    print()
-    
-    print("» __bytes__(…):")
-    print()
-    print(bytes(instance0))
-    print()
-    
-    print("» namespaces:")
-    print()
-    pprint(instance0.namespaces())
-    print()
-
-
+    test_one()
+    test_two()
 
 if __name__ == '__main__':
     test()
