@@ -6,7 +6,7 @@ from functools import partial, wraps
 
 iterchain = chain.from_iterable
 
-from clu.constants.consts import λ, φ, pytuple, QUALIFIER
+from clu.constants.consts import λ, φ, pytuple, NoDefault, QUALIFIER # type: ignore
 from clu.exporting import Exporter
 
 exporter = Exporter(path=__file__)
@@ -44,40 +44,47 @@ ismetaclass = lambda thing: hasattr(thing, '__mro__') and \
                                 len(thing.__mro__) > 1 and \
                                     thing.__mro__[-2] is type
 
-isclass = lambda thing: (thing is object) or (hasattr(thing, '__mro__') and \
-                         thing.__mro__[-1] is object and \
-                         thing.__mro__[-2] is not type)
+isclass     = lambda thing: (thing is object) or (hasattr(thing, '__mro__') and \
+                             thing.__mro__[-1] is object and \
+                             thing.__mro__[-2] is not type)
 
 isclasstype = lambda thing: hasattr(thing, '__mro__') and \
                                     thing.__mro__[-1] is object
 
-metaclass = lambda thing: ismetaclass(thing) and thing \
-                          or (isclass(thing) and type(thing) \
-                          or (type(type(thing))))
+metaclass   = lambda thing: ismetaclass(thing) and thing \
+                            or (isclass(thing) and type(thing) \
+                            or (type(type(thing))))
 
 # PREDICATE FUNCTIONS: hasattr(…) shortcuts:
 
-noattr = negate(hasattr)
-haspyattr = lambda thing, atx: hasattr(thing, f'__{atx}__')
-nopyattr = negate(haspyattr)
+hasitem     = lambda thing, itx: itx in thing
+noattr      = negate(hasattr) # type: ignore
+noitem      = negate(hasitem)
 
-anyattrs = lambda thing, *attrs: any(hasattr(thing, atx) for atx in attrs)
-allattrs = lambda thing, *attrs: all(hasattr(thing, atx) for atx in attrs)
-noattrs = negate(anyattrs)
+haspyattr   = lambda thing, atx: hasattr(thing, f'__{atx}__')
+nopyattr    = negate(haspyattr)
 
-anypyattrs = lambda thing, *attrs: any(haspyattr(thing, atx) for atx in attrs)
-allpyattrs = lambda thing, *attrs: all(haspyattr(thing, atx) for atx in attrs)
-nopyattrs = negate(anypyattrs)
+anyitems    = lambda thing, *items: any(hasitem(thing, itx) for itx in items)
+allitems    = lambda thing, *items: all(hasitem(thing, itx) for itx in items)
+noitems     = negate(anyitems)
+
+anyattrs    = lambda thing, *attrs: any(hasattr(thing, atx) for atx in attrs)
+allattrs    = lambda thing, *attrs: all(hasattr(thing, atx) for atx in attrs)
+noattrs     = negate(anyattrs)
+
+anypyattrs  = lambda thing, *attrs: any(haspyattr(thing, atx) for atx in attrs)
+allpyattrs  = lambda thing, *attrs: all(haspyattr(thing, atx) for atx in attrs)
+nopyattrs   = negate(anypyattrs)
 
 # Calling “typeof(…)” on a thing returns either its type, or – if it is
 # a classtype – the thing itself:
-typeof = lambda thing: nopyattr(thing, 'mro') and type(thing) or thing
+typeof      = lambda thing: nopyattr(thing, 'mro') and type(thing) or thing
 
 # Things with a __len__(…) method “have length”:
-haslength = lambda thing: haspyattr(thing, 'len')
+haslength   = lambda thing: haspyattr(thing, 'len')
 
 # Things with either __iter__(…) OR __getitem__(…) are iterable:
-isiterable = lambda thing: anypyattrs(thing, 'iter', 'getitem')
+isiterable  = lambda thing: anypyattrs(thing, 'iter', 'getitem')
 
 # q.v. `merge_two(…)` implementation sub. –
 # this predicate could also be called “isgetitemable()” in terms of
@@ -96,6 +103,7 @@ or_none   = lambda thing, atx: getattr(thing, atx, None)
 stor_none = lambda thing, atx: getattr_static(thing, atx, None)
 getpyattr = lambda thing, atx, default=None: getattr(thing, f'__{atx}__', default)
 getitem   = lambda thing, itx, default=None: itx in thing and thing[itx] or default
+retrieve  = lambda thing, itx, default=None: itx in thing and thing or default
 
 @export
 def resolve(thing, atx):
@@ -157,13 +165,47 @@ stattr_across = lambda atx, *things, default=tuple(): collator(stresolve, atx, *
 pyattr_across = lambda atx, *things, default=tuple(): collator(getpyattr, atx, *things, default=default)
 item_across   = lambda itx, *things, default=tuple(): collator(getitem,   itx, *things, default=default)
 
+finditem      = lambda itx, *things, default=None:    searcher(retrieve,  itx, *things, default=default)
+finditems     = lambda itx, *things, default=tuple(): collator(retrieve,  itx, *things, default=default)
+
+@export
+def try_items(itx, *things, default=NoDefault):
+    """ try_items(itx, *things[, default]) → attempt to retrieve an item
+        from each of the things, in sequence – falling back to a default,
+        or raising a KeyError if no default is specified.
+        
+        This works like “item_search(itx, *things[, default])” – with the
+        notable exception that, if any of the things are instances of
+        “collections.defaultdict”, “try_items(…)” will correctly trigger
+        any “defaultdict” instances’ default factories.
+    """
+    for thing in things:
+        try:
+            return thing[itx]
+        except KeyError:
+            pass
+    if default is not NoDefault:
+        return default
+    raise KeyError(f"{itx} not found in any of things: {things!r}")
+
+# SOME COMMON SHORTCUTS:
+
+dunder_or   = lambda thing, atx: getpyattr(thing, atx, thing)
+
+mro         = lambda thing: getpyattr(typeof(thing), 'mro')
+unwrap      = lambda thing: dunder_or(thing, 'wrapped')
+origin      = lambda thing: dunder_or(typeof(thing), 'origin')
+
+isancestor  = lambda cls, ancestor=object: isclasstype(cls) and (ancestor in mro(cls))
+isorigin    = lambda cls, original=object: isclasstype(cls) and isancestor(origin(cls), typeof(original))
+
 # ENUM PREDICATES: `isenum(…)` predicate; `enumchoices(…)` to return a tuple
 # of strings naming an enum’s choices (like duh)
 
 @export
 def isenum(cls):
     """ isenum(cls) → boolean predicate, True if cls descends from `Enum`. """
-    from clu.constants.polyfills import Enum
+    from clu.constants.polyfills import Enum # type: ignore
     if nopyattr(cls, 'mro'):
         return False
     return Enum in cls.__mro__
@@ -177,24 +219,28 @@ def enumchoices(cls):
 
 # PREDICATE LOGCIAL FUNCTIONS: all/any/and/or/xor shortcuts:
 
-predicate_nop = lambda *things: None
-function_nop = lambda iterable: None
+predicate_nop   = lambda *things: None
+function_nop    = lambda iterable: None
 
-uncallable = negate(callable)
-pyname = lambda thing: pyattr(thing, 'qualname', 'name')
-pymodule = lambda thing: pyattr(thing, 'module', 'package')
+uncallable      = negate(callable) # type: ignore
+pyname          = lambda thing: pyattr(thing, 'name', 'qualname')
+pymodule        = lambda thing: pyattr(thing, 'module', 'package')
 
-isexpandable = lambda thing: isinstance(thing, (tuple, list, set, frozenset,
-                                                map, filter, reversed) or isenum(thing))
+the_expandables = (tuple, list, set, frozenset,
+                   map, filter, reversed)
 
-isnormative = lambda thing: isinstance(thing, (str, bytes, bytearray))
-iscontainer = lambda thing: isiterable(thing) and \
-                        not isnormative(thing) and \
-                        not isclasstype(thing)
+the_normies     = (str, bytes, bytearray)
+
+isexpandable    = lambda thing: isinstance(thing, the_expandables) or isenum(thing) # type: ignore
+
+isnormative     = lambda thing: isinstance(thing, the_normies) or haspyattr(thing, 'fspath')
+iscontainer     = lambda thing: isiterable(thing) and \
+                            not isnormative(thing) and \
+                            not isclasstype(thing)
 
 # This is the equivalent of a lambda types’ built-in __repr__ function:
-lambda_repr = lambda instance, default=λ: "<function %s at 0x%0x>" % (pyname(instance) or default,
-                                                                          id(instance))
+lambda_repr     = lambda instance, default=λ: "<function %s at 0x%0x>" % (pyname(instance) or default,
+                                                                              id(instance))
 
 class Partial(partial):
     
@@ -211,7 +257,7 @@ class Partial(partial):
         try:
             super(Partial, self).__init__(*args, **kwargs)
         except:
-            super(Partial, self).__init__()
+            super(Partial, self).__init__() # type: ignore
     
     @property
     def predicate(self):
@@ -244,7 +290,7 @@ def apply_to(predicate, function, *things):
     # Ensure both the predicate and function are callable:
     if any(uncallable(f) for f in (predicate, function)):
         names = tuple(pyname(f) for f in (predicate, function))
-        raise ValueError("Noncallable passed to apply_to(%s, %s, …)" % names)
+        raise ValueError("Noncallable passed to apply_to(%s, %s, …)" % names) # type: ignore
     if len(things) < 1:
         # Return a Partial for this predicate and function:
         return Partial(apply_to, predicate, function)
@@ -286,34 +332,67 @@ case_sort = lambda c: c.lower() if c.isupper() else c.upper()
 
 # UTILITY FUNCTIONS: helpers for builtin container types:
 
+isnotnone = lambda thing: thing is not None
+
 @export
+def itervariadic(function):
+    """ Wrap a variadic function – one with the signature “function(•args)” –
+        in logic that allows it to be called with a single-argument iterable
+        (“function(iterable)”) with the same effect.
+    
+        Screens out strings, bytes-y, and file-path-ish operands without
+        iterating or expanding them.
+    """
+    @wraps(function)
+    def wrapper(*args, expand=True):
+        if len(args) == 1 and expand:
+            if isexpandable(args[0]):
+                return function(*args[0])
+            if iscontainer(args[0]):
+                return function(*tuple(args[0]))
+        return function(*args)
+    return wrapper
+
+@export
+@itervariadic
 def tuplize(*items):
     """ tuplize(*items) → Return a new tuple containing all non-`None` arguments """
     return tuple(item for item in items if item is not None)
 
 @export
+@itervariadic
 def uniquify(*items):
     """ uniquify(*items) → Return a tuple with a unique set of all non-`None` arguments """
-    return tuple(frozenset(item for item in items if item is not None))
+    seen = set() # type: set
+    stuff = []
+    for item in filter(isnotnone, items):
+        if item not in seen:
+            seen.add(item)
+            stuff.append(item)
+    return tuple(stuff)
 
 @export
+@itervariadic
 def listify(*items):
     """ listify(*items) → Return a new list containing all non-`None` arguments """
-    return list(item for item in items if item is not None)
+    return [item for item in items if item is not None]
 
 # UTILITY FUNCTIONS: helpers for builtin predicate functions over iterables:
 
 @export
+@itervariadic
 def allof(*items):
     """ allof(*items) → Return the result of “all(…)” on all non-`None` arguments """
     return all(item for item in items if item is not None)
 
 @export
+@itervariadic
 def anyof(*items):
     """ anyof(*items) → Return the result of “any(…)” on all non-`None` arguments """
     return any(item for item in items if item is not None)
 
 @export
+@itervariadic
 def noneof(*items):
     """ noneof(*items) → Return the result of “not any(…)” on all non-`None` arguments """
     return negate(any)(item for item in items if item is not None)
@@ -338,13 +417,21 @@ export(isclasstype,     name='isclasstype',     doc="isclasstype(thing) → bool
 export(metaclass,       name='metaclass',       doc="metaclass(thing) → Returns: a) thing, if thing is a metaclass; b) type(thing), if thing is a class; or c) type(type(thing)), for all other instances")
 export(typeof,          name='typeof',          doc="typeof(thing) → Returns thing, if thing is a class type; or type(thing), if it is not")
 
+export(hasitem,         name='hasitem',         doc="hasitem(thing, item) → boolean predicate, shortcut for `item in thing`")
 export(noattr,          name='noattr',          doc="noattr(thing, attribute) → boolean predicate, shortcut for `(not hasattr(thing, attribute))`")
+export(noitem,          name='noitem',          doc="noitem(thing, item) → boolean predicate, shortcut for `(not hasitem(thing, item))`")
+
 export(haspyattr,       name='haspyattr',       doc="haspyattr(thing, attribute) → boolean predicate, shortcut for `hasattr(thing, '__%s__' % attribute)`")
 export(nopyattr,        name='nopyattr',        doc="nopyattr(thing, attribute) → boolean predicate, shortcut for `(not hasattr(thing, '__%s__' % attribute))`")
 
+export(anyitems,        name='anyitems',        doc="anyitems(thing, *items) → boolean predicate, shortcut for `any(hasitem(thing, itx) for itx in items)`")
+export(allitems,        name='allitems',        doc="allitems(thing, *items) → boolean predicate, shortcut for `all(hasitem(thing, itx) for itx in items)`")
+export(noitems,         name='noitems',         doc="noitems(thing, *items) → boolean predicate, shortcut for `(not anyitems(*attributes)`")
+
 export(anyattrs,        name='anyattrs',        doc="anyattrs(thing, *attributes) → boolean predicate, shortcut for `any(hasattr(thing, atx) for atx in attributes)`")
 export(allattrs,        name='allattrs',        doc="allattrs(thing, *attributes) → boolean predicate, shortcut for `all(hasattr(thing, atx) for atx in attributes)`")
-export(noattrs,         name='noattrs',         doc="noattrs(thing, *attributes) → boolean predicate, shortcut for `(not anypyattrs(hasattr(thing, atx) for atx in attributes))`")
+export(noattrs,         name='noattrs',         doc="noattrs(thing, *attributes) → boolean predicate, shortcut for `(not anypyattrs(*attributes)`")
+
 export(anypyattrs,      name='anypyattrs',      doc="anypyattrs(thing, *attributes) → boolean predicate, shortcut for `any(haspyattr(thing, atx) for atx in attributes)`")
 export(allpyattrs,      name='allpyattrs',      doc="allpyattrs(thing, *attributes) → boolean predicate, shortcut for `all(haspyattr(thing, atx) for atx in attributes)`")
 export(nopyattrs,       name='nopyattrs',       doc="nopyattrs(thing, *attributes) → boolean predicate, shortcut for `(not any(haspyattr(thing, atx) for atx in attributes))`")
@@ -361,6 +448,7 @@ export(or_none,         name='or_none',         doc="or_none(thing, attribute) 
 export(stor_none,       name='stor_none',       doc="stor_none(thing, attribute) → shortcut for `inspect.getattr_static(thing, attribute, None)`")
 export(getpyattr,       name='getpyattr',       doc="getpyattr(thing, attribute[, default]) → shortcut for `getattr(thing, '__%s__' % attribute[, default])`")
 export(getitem,         name='getitem',         doc="getitem(thing, item[, default]) → shortcut for `thing.get(item[, default])`")
+export(retrieve,        name='retrieve',        doc="retrieve(thing, item[, default]) → shortcut for `hasitem(thing, itx) and thing or default`")
 
 export(accessor,        name='accessor',        doc="accessor(func, thing, *attributes) → return the first non-None value had by successively applying func(thing, attribute) to all attributes")
 export(acquirer,        name='acquirer',        doc="acquirer(func, thing, *attributes) → return all of the non-None values had by successively applying func(thing, attribute) to all attributes")
@@ -387,6 +475,16 @@ export(stattr_across,   name='stattr_across',   doc="stattr_across(attribute, *t
 export(pyattr_across,   name='pyattr_across',   doc="pyattr_across(attribute, *things) → Return all of the existing named __special__ attributes across all things (given 1+ things)")
 export(item_across,     name='item_across',     doc="item_across(attribute, *things) → Return all of the existing named items held across all things (given 1+ things)")
 
+export(finditem,        name='finditem',        doc="finditem(itx, *mappings, default=None) → Return the first mapping that contains “itx”, or “default” if “itx” isn’t found in any of them")
+export(finditems,       name='finditems',       doc="finditems(itx, *mappings, default=tuple()) → Return a tuple of all mappings that contain “itx”, or “default” if “itx” isn’t found in any of them")
+
+export(dunder_or,       name='dunder_or',       doc="dunder_or(thing, atx) → Like “getpyattr(…)” only with a default return value of `thing` itself")
+export(mro,             name='mro',             doc="mro(thing) → Return the method resolution order (née “MRO”) tuple for thing (using “type(thing)” for non-classtype operands)")
+export(unwrap,          name='unwrap',          doc="unwrap(thing) → Return either `thing.__wrapped__` or `thing` for a given `thing`")
+export(origin,          name='origin',          doc="origin(thing) → Return either `typeof(thing).__origin__` or `typeof(thing)` for a given `thing`")
+export(isancestor,      name='isancestor',      doc="isancestor(thing, ancestor=object) → boolean predicate, True if `ancestor` is found in “mro(thing)”")
+export(isorigin,        name='isorigin',        doc="isorigin(thing, original=object) → boolean predicate, True if `original` is an ancestor of “origin(thing)”")
+
 export(predicate_nop,   name='predicate_nop',   doc="predicate_nop(thing) → boolean predicate that always returns `None`")
 export(function_nop,    name='function_nop',    doc="function_nop(*args) → variadic function always returns `None`")
 export(uncallable,      name='uncallable',      doc="uncallable(thing) → boolean predicate, shortcut for `not callable(thing)`")
@@ -407,6 +505,7 @@ export(predicate_or,    name='predicate_or',    doc="predicate_or(predicate, a,
 export(predicate_xor,   name='predicate_xor',   doc="predicate_xor(predicate, a, b) → boolean predicate, shortcut for `apply_to(predicate, any, a, b) and not apply_to(predicate, all, a, b)`")
 
 export(case_sort,       name='case_sort',       doc="case_sort(string) → Sorting predicate to sort UPPERCASE names first")
+export(isnotnone,       name='isnotnone',       doc="isnotnone(thing) → boolean predicate, return True if “thing” is not None")
 
 export(thing_has,       name='thing_has',       doc="thing_has(thing, attribute) → boolean predicate, True if `thing` has “attribute” (in either `__dict__` or `__slots__`)")
 export(class_has,       name='class_has',       doc="class_has(cls, attribute) → boolean predicate, True if `cls` is a class type and has “attribute” (in either `__dict__` or `__slots__`)")
