@@ -63,6 +63,18 @@ from clu.exporting import (doctrim,
                            Exporter, ExporterBase, Registry,
                            path_to_dotpath)
 
+from clu.extending import (Extensible,
+                           pairtype, ΩΩ, pairmro, ω, pair, Ω,
+                           DoubleDutchRegistry, doubledutch,
+                           DoubleDutchFunction)
+
+from clu.importing import (Module, Finder, Loader,
+                           initialize_types,
+                           ModuleBase, FinderBase, LoaderBase,
+                           Package, ModuleSpec,
+                           Registry as ClassModuleRegistry,
+                           ArgumentSink)
+
 from clu.sanitizer import sanitize, sanitizers, utf8_encode, utf8_decode
 from clu.version import version_info
 
@@ -76,12 +88,16 @@ from clu.predicates import (negate,
                             isiterable, ismergeable,
                             always, never, nuhuh,
                             no_op, or_none, stor_none, resolve,
-                            getpyattr, getitem,
+                            getpyattr, getitem, retrieve,
                             accessor, acquirer, collator, searcher,
                             attr, stattr, pyattr, item,
                             attrs, stattrs, pyattrs, items,
                             attr_search, stattr_search, pyattr_search, item_search,
                             attr_across, stattr_across, pyattr_across, item_across,
+                            finditem, finditems,
+                            try_items,
+                            dunder_or, mro, unwrap, origin,
+                            isancestor, isorigin,
                             isenum, enumchoices,
                             predicate_nop, function_nop, uncallable,
                             pyname, pymodule,
@@ -91,15 +107,18 @@ from clu.predicates import (negate,
                             predicate_all, predicate_any, predicate_none,
                             predicate_and, predicate_or, predicate_xor,
                             thing_has, class_has,
-                            isslotted, isdictish, isslotdicty, slots_for,
+                            isslotted, isdictish, isslotdicty,
                             case_sort,
+                            itervariadic,
                             tuplize, uniquify, listify,
-                            allof, anyof, noneof)
+                            allof, anyof, noneof,
+                            slots_for)
 
 from clu.typespace import SimpleNamespace, Namespace, types, modulize
 
-from clu.typology import (samelength, differentlength, isunique, istypelist, maketypelist,
-                          isderivative, subclasscheck, graceful_issubclass,
+from clu.typology import (samelength, differentlength, isunique,
+                          istypelist, ismetatypelist, maketypelist, makemetatypelist,
+                          isderivative, subclasscheck, metaclasscheck, graceful_issubclass,
                           numeric_types, array_types, scalar_types, string_types, bytes_types,
                           path_classes, path_types, file_types,
                           dict_types, mapping_types, mapping_classes,
@@ -107,11 +126,12 @@ from clu.typology import (samelength, differentlength, isunique, istypelist, mak
                           ispathtype, ispath, isnotpath, isvalidpath,
                           isaliasdescriptor, hasmembers, hasaliases,
                           isabstractmethod, isabstract, isabstractcontextmanager, iscontextmanager,
+                          isabc, isslottedtype, isextensibletype,
                           isnumber, isnumeric, iscomplex, ismapping,
                           isarray, isscalar, isstring, isbytes, ismodule,
                           isfunction, ΛΛ, islambda, λλ,
                           iscallable, iscallabletype, ishashable, issequence,
-                          isxlist, isxtypelist,
+                          isxlist, isxtypelist, isxmetatypelist, isabclist,
                           ispathtypelist, ispathlist, isvalidpathlist,
                           isnumberlist, isnumericlist, iscomplexlist, ismappinglist,
                           isarraylist, isscalarlist, isstringlist, isbyteslist,
@@ -119,13 +139,16 @@ from clu.typology import (samelength, differentlength, isunique, istypelist, mak
                           iscallablelist, iscallabletypelist, ishashablelist, issequencelist)
 
 from clu.naming import (determine_module, nameof, moduleof,
+                        isbuiltin, isnativemodule, isnative,
                         dotpath_join, dotpath_split, qualified_import,
                                                      qualified_name_tuple,
                                                      qualified_name,
                         dotpath_to_prefix, path_to_prefix,
                         split_abbreviations)
 
-from clu.config.abc import NAMESPACE_SEP, NamespacedMutableMapping, Cloneable, FlatOrderedSet
+from clu.abstract import Cloneable, ReprWrapper
+
+from clu.config.abc import NAMESPACE_SEP, FlatOrderedSet, NamespacedMutableMapping
 from clu.config.base import Flat, Nested
 from clu.config.env import Env
 from clu.config.filebase import FileName, FileBase
@@ -138,7 +161,7 @@ from clu.config.formats import JsonFile, PickleFile, TomlFile, YamlFile
 
 from clu.csv import pad_csv
 from clu.dicts import OrderedMappingView, OrderedItemsView, OrderedKeysView, OrderedValuesView
-from clu.dicts import merge_two, merge_as, merge, asdict
+from clu.dicts import ChainMap, merge_two, merge_as, merge, asdict
 
 # Add miscellaneous necessities:
 from PIL import Image
@@ -184,8 +207,8 @@ from clu.repl.ansi import (print_separator, evict_announcer,
 from clu.repl.banners import print_banner
 
 # Compile all Greekly definitions and their names:
-GREEK_STRINGS = ('σ', 'Σ', 'λ', 'λλ', 'Λ', 'ΛΛ', 'φ')
-GREEK_DEFS = (σ, Σ, λ, λλ, Λ, ΛΛ, φ)
+GREEK_STRINGS = ('σ', 'Σ', 'λ', 'λλ', 'Λ', 'ΛΛ', 'φ', 'Ω', 'ω', 'ΩΩ')
+GREEK_DEFS = (σ, Σ, λ, λλ, Λ, ΛΛ, φ, Ω, ω, ΩΩ)
 
 GREEK_PHONETICS = ('sigma-lower',
                    'sigma-upper',
@@ -193,7 +216,10 @@ GREEK_PHONETICS = ('sigma-lower',
                    'double-lambda-lower',
                    'lambda-upper',
                    'double-lambda-upper',
-                   'phi-lower')
+                   'phi-lower',
+                   'omega-upper',
+                   'omega-lower'
+                   'double-omega-upper')
 
 GREEK_STRINGDICT = dict(zip(GREEK_STRINGS, GREEK_DEFS))
 GREEK_NAMEDICT = dict(zip(GREEK_PHONETICS, GREEK_DEFS))
@@ -287,6 +313,16 @@ __all__ = ('Image',
            'sysmods',
            'Exporter', 'ExporterBase', 'Registry',
            'path_to_dotpath',
+           'Extensible',
+           'pairtype', 'ΩΩ', 'pairmro', 'ω', 'pair', 'Ω',
+           'DoubleDutchRegistry', 'doubledutch',
+           'DoubleDutchFunction',
+           'Module', 'Finder', 'Loader',
+           'initialize_types',
+           'ModuleBase', 'FinderBase', 'LoaderBase',
+           'Package', 'ModuleSpec',
+           'ClassModuleRegistry',
+           'ArgumentSink',
            'negate',
            'ismetaclass', 'isclass', 'isclasstype',
            'metaclass', 'typeof',
@@ -297,12 +333,16 @@ __all__ = ('Image',
            'isiterable', 'ismergeable',
            'always', 'never', 'nuhuh',
            'no_op', 'or_none', 'stor_none', 'resolve',
-           'getpyattr', 'getitem',
+           'getpyattr', 'getitem', 'retrieve',
            'accessor', 'acquirer', 'collator', 'searcher',
            'attr', 'stattr', 'pyattr', 'item',
            'attrs', 'stattrs', 'pyattrs', 'items',
            'attr_search', 'stattr_search', 'pyattr_search', 'item_search',
            'attr_across', 'stattr_across', 'pyattr_across', 'item_across',
+           'finditem', 'finditems',
+           'try_items',
+           'dunder_or', 'mro', 'unwrap', 'origin',
+           'isancestor', 'isorigin',
            'isenum', 'enumchoices',
            'predicate_nop', 'function_nop', 'uncallable',
            'pyname', 'pymodule',
@@ -312,13 +352,16 @@ __all__ = ('Image',
            'predicate_all', 'predicate_any', 'predicate_none',
            'predicate_and', 'predicate_or', 'predicate_xor',
            'thing_has', 'class_has',
-           'isslotted', 'isdictish', 'isslotdicty', 'slots_for',
+           'isslotted', 'isdictish', 'isslotdicty',
+           'slots_for',
            'case_sort',
+           'itervariadic',
            'tuplize', 'uniquify', 'listify',
            'allof', 'anyof', 'noneof',
            'SimpleNamespace', 'Namespace', 'types', 'modulize',
-           'samelength', 'differentlength', 'isunique', 'istypelist', 'maketypelist',
-           'isderivative', 'subclasscheck', 'graceful_issubclass',
+           'samelength', 'differentlength', 'isunique',
+           'istypelist', 'ismetatypelist', 'maketypelist', 'makemetatypelist',
+           'isderivative', 'subclasscheck', 'metaclasscheck', 'graceful_issubclass',
            'numeric_types', 'array_types', 'scalar_types', 'string_types', 'bytes_types',
            'path_classes', 'path_types', 'file_types',
            'dict_types', 'mapping_types', 'mapping_classes',
@@ -326,22 +369,24 @@ __all__ = ('Image',
            'ispathtype', 'ispath', 'isnotpath', 'isvalidpath',
            'isaliasdescriptor', 'hasmembers', 'hasaliases',
            'isabstractmethod', 'isabstract', 'isabstractcontextmanager', 'iscontextmanager',
+           'isabc', 'isslottedtype', 'isextensibletype',
            'isnumber', 'isnumeric', 'iscomplex', 'ismapping',
            'isarray', 'isscalar', 'isstring', 'isbytes',
            'ismodule', 'isfunction', 'ΛΛ', 'islambda', 'λλ',
            'iscallable', 'iscallabletype', 'ishashable', 'issequence',
-           'isxlist', 'isxtypelist',
+           'isxlist', 'isxtypelist', 'isxmetatypelist', 'isabclist',
            'ispathtypelist', 'ispathlist', 'isvalidpathlist',
            'isnumberlist', 'isnumericlist', 'iscomplexlist', 'ismappinglist',
            'isarraylist', 'isscalarlist', 'isstringlist', 'isbyteslist',
            'ismodulelist', 'isfunctionlist', 'islambdalist',
            'iscallablelist', 'iscallabletypelist', 'ishashablelist', 'issequencelist',
            'determine_module', 'nameof', 'moduleof',
+           'isbuiltin', 'isnativemodule', 'isnative',
            'dotpath_join', 'dotpath_split',
            'qualified_import', 'qualified_name_tuple', 'qualified_name',
            'dotpath_to_prefix', 'path_to_prefix',
            'split_abbreviations',
-           'NAMESPACE_SEP', 'NamespacedMutableMapping', 'Cloneable', 'FlatOrderedSet',
+           'NAMESPACE_SEP', 'Cloneable', 'ReprWrapper', 'FlatOrderedSet', 'NamespacedMutableMapping',
            'Flat', 'Nested',
            'Env', 'FileName', 'FileBase',
            'ValidationError', 'hoist',
@@ -350,7 +395,7 @@ __all__ = ('Image',
            'Schema', 'JsonFile', 'PickleFile', 'TomlFile', 'YamlFile',
            'pad_csv',
            'OrderedMappingView', 'OrderedItemsView', 'OrderedKeysView', 'OrderedValuesView',
-           'merge_two', 'merge_as', 'merge', 'asdict',
+           'ChainMap', 'merge_two', 'merge_as', 'merge', 'asdict',
            'DUNDER', 'SUNDER', 'alias', 'AliasingEnumMeta', 'AliasingEnum',
            'print_separator', 'evict_announcer',
            'print_ansi', 'print_ansi_centered',
@@ -377,6 +422,14 @@ else:
     # Extend `__all__`:
     __all__ += ('colorio', 'colormath')
 
+try:
+    import pout
+except (ImportError, SyntaxError):
+    pass
+else:
+    # Extend `__all__`:
+    __all__ += ('pout',)
+
 try:
     import pytz
     import dateutil
