@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
 
-import array
+import abc
 import argparse
 import collections
 import collections.abc
@@ -11,16 +11,18 @@ import io
 import operator
 import os
 
-from clu.constants.consts import λ, φ
+from clu.constants.consts import λ, φ # type: ignore
 from clu.constants.polyfills import long, unicode, numpy
-from clu.constants.polyfills import Path
+from clu.constants.polyfills import Path # type: ignore
 from clu.enums import alias
-from clu.exporting import Exporter
+from clu.extending import Extensible
+from clu.exporting import Slotted, Exporter
 
-from clu.predicates import (isclasstype, isenum,
+from clu.predicates import (negate,
+                            ismetaclass, isclasstype, metaclass,
                             allpyattrs, haspyattr, nopyattr,
-                            isiterable, haslength,
-                            getpyattr, or_none,
+                            isiterable, haslength, typeof,
+                            getpyattr, or_none, isenum,
                             pyattr, attrs,
                             tuplize, uniquify,
                             apply_to, predicate_any,
@@ -37,9 +39,11 @@ export = exporter.decorator()
 samelength = lambda a, b: haslength(a) and haslength(b) and operator.eq(len(a), len(b))
 differentlength = lambda a, b: haslength(a) and haslength(b) and operator.ne(len(a), len(b))
 isunique = lambda thing: isiterable(thing) and samelength(tuple(thing), frozenset(thing))
+
 istypelist = predicate_all(isclasstype)
-maketypelist = apply_to(lambda thing: isclasstype(thing) and thing or type(thing),
-                        lambda total: tuple(frozenset(total)))
+ismetatypelist = predicate_all(ismetaclass)
+maketypelist = apply_to(typeof, uniquify)
+makemetatypelist = apply_to(metaclass, uniquify)
 
 @export
 def isderivative(putative, thing):
@@ -51,7 +55,7 @@ def isderivative(putative, thing):
     """
     try:
         return issubclass(putative, thing)
-    except TypeError:
+    except (AttributeError, TypeError):
         return isinstance(putative, thing)
 
 # PREDICATE FUNCTION “subclasscheck”: A wrapper for `issubclass(…)` and
@@ -60,7 +64,11 @@ def isderivative(putative, thing):
 
 subclasscheck = lambda putative, *thinglist: predicate_any(
                 lambda thing: isderivative(putative, thing),
-                                                *thinglist)
+                                                    *thinglist)
+
+metaclasscheck = lambda putative, *thinglist: predicate_any(
+                 lambda thing: isderivative(metaclass(putative), thing),
+                                                                *thinglist)
 
 # LEGACY CODE SUPPORT:
 graceful_issubclass = subclasscheck
@@ -69,7 +77,7 @@ graceful_issubclass = subclasscheck
 # and other similar stuff.
 
 numeric_types = uniquify(bool, int, long, float, complex, decimal.Decimal)
-array_types = (array.ArrayType, bytearray, memoryview)
+array_types = (types.Array, bytearray, memoryview) # type: tuple
 array_types += attrs(numpy, 'ndarray',
                             'matrix',
                             'ma.core.MaskedArray')
@@ -86,7 +94,7 @@ except (ImportError, SyntaxError):
 
 bytes_types = (bytes, bytearray)
 path_classes = tuplize(argparse.FileType, or_none(os, 'PathLike'), Path) # Path may be “None” in disguise
-path_types = string_types + bytes_types + path_classes
+path_types = string_types + bytes_types + path_classes # type: ignore
 file_types = (io.TextIOBase, io.BufferedIOBase, io.RawIOBase, io.IOBase)
 
 dict_types      = { dict, collections.defaultdict,
@@ -125,8 +133,9 @@ callable_types += attrs(types, 'Coroutine',
 # of the aforementioned typelists:
 
 # Path types:
-ispathtype = lambda cls: issubclass(cls, path_types)
-ispath = lambda thing: subclasscheck(thing, path_types) or haspyattr(thing, 'fspath')
+ispathtype  = lambda cls: issubclass(cls, path_types)
+ispath      = lambda thing: subclasscheck(thing, path_types) or haspyattr(thing, 'fspath')
+isnotpath   = negate(ispath)
 isvalidpath = lambda thing: ispath(thing) and os.path.exists(os.path.expanduser(thing))
 
 # Abstract items and context managers:
@@ -147,6 +156,10 @@ iskeysview   = lambda thing: isinstance(thing,    collections.abc.KeysView)
 isvaluesview = lambda thing: isinstance(thing,    collections.abc.ValuesView)
 isitemsview  = lambda thing: isinstance(thing,    collections.abc.ItemsView)
 
+isabc = lambda thing: metaclasscheck(thing, abc.ABCMeta)
+isslottedtype = lambda thing: metaclasscheck(thing, Slotted)
+isextensibletype = lambda thing: metaclasscheck(thing, Extensible)
+
 # Typelist predicates:
 isnumber = lambda thing: subclasscheck(thing, numeric_types)
 isnumeric = lambda thing: subclasscheck(thing, numeric_types)
@@ -157,15 +170,19 @@ isscalar = lambda thing: subclasscheck(thing, scalar_types)
 isstring = lambda thing: subclasscheck(thing, string_types)
 isbytes = lambda thing: subclasscheck(thing, bytes_types)
 ismodule = lambda thing: subclasscheck(thing, types.Module)
-isfunction = ΛΛ = lambda thing: isinstance(thing, Λ) and not isclasstype(thing)
+isfunction = ΛΛ = lambda thing: isinstance(thing, Λ) and nopyattr(thing, 'mro')
 islambda = λλ = lambda thing: pyattr(thing, 'lambda_name', 'name', 'qualname') in (λ, φ)
 iscallable = lambda thing: haspyattr(thing, 'call') and nopyattr(thing, 'code')
+iscallabletype = lambda thing: isinstance(thing, callable_types) and nopyattr(thing, 'mro')
 issequence = lambda thing: isinstance(thing, collections.abc.Sequence)
 ishashable = lambda thing: isinstance(thing, collections.abc.Hashable)
 
 # Helper predicates for composing sequence-based predicates:
 isxlist = lambda predicate, thinglist: issequence(thinglist) and predicate_all(predicate, thinglist)
 isxtypelist = lambda predicate, thinglist: istypelist(thinglist) and predicate_all(predicate, thinglist)
+isxmetatypelist = lambda predicate, thinglist: ismetatypelist(thinglist) and predicate_all(predicate, thinglist)
+
+isabclist = lambda thinglist: isxtypelist(lambda thing: metaclasscheck(thing, abc.ABCMeta), thinglist)
 
 # Typelist list-type predicates (?!)
 ispathtypelist = predicate_all(lambda thing: isclasstype(thing) and ispathtype(thing))
@@ -184,6 +201,7 @@ ismodulelist = predicate_all(ismodule)
 isfunctionlist = predicate_all(ΛΛ)
 islambdalist = predicate_all(λλ)
 iscallablelist = predicate_all(iscallable)
+iscallabletypelist = predicate_all(iscallabletype)
 issequencelist = predicate_all(issequence)
 ishashablelist = predicate_all(ishashable)
 
@@ -192,15 +210,23 @@ export(samelength,      name='samelength',  doc="samelength(a, b) → boolean pr
 export(differentlength, name='differentlength', doc="differentlength(a, b) → boolean predicate, True if both `len(a)` and `len(b)` are defined, but are unequal")
 export(isunique,        name='isunique',    doc="isunique(thing) → boolean predicate, True if `thing` is an iterable with unique contents")
 export(istypelist,      name='istypelist',  doc="istypelist(thing) → boolean predicate, True if `thing` is a “typelist” – a list consisting only of class types")
+export(ismetatypelist,  name='ismetatypelist',
+                         doc="ismetatypelist(thing) → boolean predicate, True if `thing` is a “metatypelist” – a list consisting only of metaclass types")
 export(maketypelist,    name='maketypelist',
                          doc="maketypelist(iterable) → convert an iterable of unknown things into a uniquified typelist – a list consisting only of class types")
+export(makemetatypelist, name='makemetatypelist',
+                         doc="makemetatypelist(iterable) → convert an iterable of unknown things into a uniquified metatypelist – a list consisting only of metaclass types")
 
 export(subclasscheck,   name='subclasscheck',
-                         doc="subclasscheck(putative, *cls_or_tuple) → A wrapper for `issubclass(…)` and `isinstance(…)` that tries to work with you`")
+                         doc="subclasscheck(putative, *cls_or_tuple) → A wrapper for `issubclass(…)` and `isinstance(…)` that tries to work with you")
+
+export(metaclasscheck,  name='metaclasscheck',
+                         doc="metaclasscheck(putative, *cls_or_tuple) → A wrapper for `issubclass(…)` and `isinstance(…)` that specifically inspects the metaclass of its primary operand")
 
 export(graceful_issubclass,
                         name='graceful_issubclass')
 
+
 # NO DOCS ALLOWED:
 export(numeric_types,   name='numeric_types')
 export(array_types,     name='array_types')
@@ -218,7 +244,8 @@ export(Λ,               name='Λ')
 export(callable_types,  name='callable_types')
 
 export(ispathtype,      name='ispathtype',  doc="ispathtype(thing) → boolean predicate, True if `thing` is a path type")
-export(ispath,          name='ispath',      doc="ispath(thing) → boolean predicate, True if `thing` seems to be path-ish instance")
+export(ispath,          name='ispath',      doc="ispath(thing) → boolean predicate, True if `thing` seems to be a path-ish instance")
+export(isnotpath,       name='isnotpath',   doc="isnotpath(thing) → boolean predicate, True if `thing` seems not to be a path-ish instance")
 export(isvalidpath,     name='isvalidpath', doc="isvalidpath(thing) → boolean predicate, True if `thing` represents a valid path on the filesystem")
 
 export(isabstractmethod,                    doc="isabstractmethod(thing) → boolean predicate, True if `thing` is a method declared “abstract” with `@abc.abstractmethod`")
@@ -236,6 +263,11 @@ export(iskeysview,      name='iskeysview',      doc="iskeysview(thing) → boole
 export(isvaluesview,    name='isvaluesview',    doc="isvaluesview(thing) → boolean predicate, True if `thing` is a mapping-values view instance")
 export(isitemsview,     name='isitemsview',     doc="isitemsview(thing) → boolean predicate, True if `thing` is a mapping-items view instance")
 
+export(isabc,           name='isabc',           doc="isabc(thing) → boolean predicate, True if `thing` is an abstract base class (née ‘ABC’) or a descendant or instance of same")
+export(isslottedtype,   name='isslottedtype',   doc="isslottedtype(thing) → boolean predicate, True if `thing` has “clu.exporting.Slotted” as its metaclass")
+export(isextensibletype,    name='isextensibletype',
+                             doc="isextensibletype(thing) → boolean predicate, True if `thing` is an “extensible” type (q.v. “clu.extending” source supra.)")
+
 export(isnumber,        name='isnumber',    doc="isnumber(thing) → boolean predicate, True if `thing` is a numeric type or an instance of same")
 export(isnumeric,       name='isnumeric',   doc="isnumeric(thing) → boolean predicate, True if `thing` is a numeric type or an instance of same")
 export(iscomplex,       name='iscomplex',   doc="iscomplex(thing) → boolean predicate, True if `thing` is a complex numeric type or an instance of same")
@@ -251,11 +283,15 @@ export(islambda,        name='islambda',    doc="islambda(thing) → boolean pre
 export(λλ,              name='λλ',          doc="λλ(thing) → boolean predicate, True if `thing` is a function created with the «lambda» keyword")
 export(iscallable,      name='iscallable',  doc="iscallable(thing) → boolean predicate, True if `thing` is a callable type (a class with a “__call__” method) or an instance of same\n\n"
                                                 "N.B. this predicate is *NOT* the same as the built-in “callable(…)” predicate")
+export(iscallabletype,  name='iscallabletype',  doc="iscallabletype(thing) → boolean predicate, True if `thing` is one of the predefined callable types (q.v. “clu.typology.callable_types” supra.)")
 export(ishashable,      name='ishashable',  doc="ishashable(thing) → boolean predicate, True if `thing` can be hashed, via the builtin `hash(…)` function")
 export(issequence,      name='issequence',  doc="issequence(thing) → boolean predicate, True if `thing` is a sequence type (e.g. a `tuple` or `list` type)")
 
 export(isxlist,         name='isxlist',     doc="isxlist(predicate, thinglist) → boolean predicate, True if `thinglist` is a sequence of items, all of which match the predicate function")
 export(isxtypelist,     name='isxtypelist', doc="isxtypelist(predicate, thinglist) → boolean predicate, True if `thinglist` is a typelist, all types in which match the predicate function")
+export(isxmetatypelist, name='isxmetatypelist',
+                         doc="isxmetatypelist(predicate, thinglist) → boolean predicate, True if `thinglist` is a metatypelist, all types in which match the predicate function")
+export(isabclist,       name='isabclist' ,      doc="isabclist(thinglist) → boolean predicate, True if `thinglist` is a sequence of abstract base classes (née ABCs)")
 
 export(ispathtypelist,  name='ispathtypelist',  doc="ispathtypelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of path-related class types")
 export(ispathlist,      name='ispathlist',      doc="ispathlist(thinglist) → boolean predicate, True if `thinglist` is a sequence of path-like instances")
@@ -273,6 +309,7 @@ export(ismodulelist,    name='ismodulelist',    doc="ismodulelist(thinglist) →
 export(isfunctionlist,  name='isfunctionlist',  doc="isfunctionlist(thinglist) → boolean predicate, True if `thinglist` is a sequence of callable function types")
 export(islambdalist,    name='islambdalist',    doc="islambdalist(thinglist) → boolean predicate, True if `thinglist` is a sequence of functions created with the «lambda» keyword")
 export(iscallablelist,  name='iscallablelist',  doc="iscallablelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of callable types (class types with “__call__” methods or instances of same)")
+export(iscallabletypelist,  name='iscallabletypelist',  doc="iscallabletypelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of predefined callable types (q.v. “clu.typology.callable_types” supra.)")
 export(issequencelist,  name='issequencelist',  doc="issequencelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of sequence types")
 export(ishashablelist,  name='ishashablelist',  doc="ishashablelist(thinglist) → boolean predicate, True if `thinglist` is a sequence of things that can be hashed, via the builtin `hash(…)` function")
 
