@@ -15,7 +15,6 @@ class TestConfig(object):
         pypath.remove_invalid_paths()  # cleans “sys.path”
         pypath.append_paths(prefix)    # extends “sys.path”
         
-        # from yodogg.config import Env, JsonFile, TomlFile
         from yodogg.config import Env
         from yodogg.config import MySchema
         
@@ -30,11 +29,11 @@ class TestConfig(object):
         assert 'YODOGG_METADATA_AUTHOR' not in environment
         assert 'YODOGG_METADATA_COPYRIGHT' not in environment
         
-        assert 'YODOGG_YODOGG_YODOGG' not in environment
         assert 'YODOGG_YODOGG_IHEARD' not in environment
+        assert 'YODOGG_YODOGG_YOULIKE' not in environment
         assert 'YODOGG_YODOGG_ANDALSO' not in environment
         
-        env.update(schema.nestify(stringify=True))
+        env.update(schema.nestify(stringify=False))
         
         assert environment['YODOGG_TITLE'] == "YoDoggApp"
         assert environment['YODOGG_VERSION'] == "1"
@@ -42,13 +41,12 @@ class TestConfig(object):
         
         assert environment['YODOGG_METADATA_RELEASEDATE']
         assert environment['YODOGG_METADATA_AUTHOR'] == "Alexander Böhn"
-        # assert environment['YODOGG_METADATA_CONSIDERATIONS'] is None
         assert 'YODOGG_METADATA_CONSIDERATIONS' not in environment
         assert environment['YODOGG_METADATA_COPYRIGHT']
         
-        assert environment['YODOGG_YODOGG_YODOGG'] == "Yo dogg,"
-        assert environment['YODOGG_YODOGG_IHEARD'] == "I heard"
-        assert environment['YODOGG_YODOGG_ANDALSO'] == "and: YoDoggApp"
+        assert environment['YODOGG_YODOGG_IHEARD'] == "…I heard"
+        assert environment['YODOGG_YODOGG_YOULIKE'] == "you like:"
+        # assert environment['YODOGG_YODOGG_ANDALSO'] == "and: YoDoggApp"
     
     def test_package_schema(self, dirname):
         from datetime import datetime
@@ -77,9 +75,9 @@ class TestConfig(object):
         assert schema.copyright.startswith("YoDoggApp")
         
         # NAMESPACE: yodogg
-        assert schema.yodogg        == "Yo dogg,"
-        assert schema.iheard        == "I heard"
-        assert schema.andalso       == "and: YoDoggApp"
+        # assert schema.yodogg.iheard        == "…I heard"
+        assert schema.youlike       == "you like:"
+        # assert schema.andalso       == "and: YoDoggApp"
     
     def test_nested_and_flat(self):
         from clu.config.base import Nested
@@ -134,7 +132,7 @@ class TestConfig(object):
         assert renestified == nested
     
     def test_FlatOrderedSet(self):
-        from clu.config.fieldtypes import FlatOrderedSet
+        from clu.config.abc import FlatOrderedSet
         
         stuff = FlatOrderedSet(None, "a", "b", FlatOrderedSet("c", None, "a", "d"))
         summary = FlatOrderedSet("a", "b", "c", "d")
@@ -156,9 +154,6 @@ class TestConfig(object):
             del yodogg
         assert "cannot import name" in str(exc.value)
     
-    def test_dict_merge(self):
-        pass
-    
     def test_env_get(self, environment):
         from clu.config.env import Env
         env = Env()
@@ -211,6 +206,51 @@ class TestConfig(object):
         
         assert 'envtest0' not in env.namespaces()
     
+    def test_toml_and_file_direct(self, dirname):
+        from clu.config.formats import TomlFile
+        
+        cfgs = dirname.subdirectory('data').subdirectory('config')
+        
+        assert cfgs.exists
+        assert TomlFile.appname in TomlFile.filename
+        assert TomlFile.filename in cfgs
+        
+        # Instantiate a TomlFile by direct filesystem path:
+        toml_path = cfgs[TomlFile.filename]
+        toml_file = TomlFile(toml_path)
+        
+        assert os.path.exists(toml_file.filepath)
+        assert toml_file.filesuffix == 'toml'
+        assert toml_file.filepath.endswith(toml_file.filesuffix)
+        
+        assert toml_file.namespaces() == ('debugging', 'userinfo')
+        assert toml_file['project'] == 'clu'
+        assert toml_file['description'] is not None
+        assert toml_file['description_content_type'] == 'text/markdown'
+        assert toml_file['debugging:debug'] == True
+        assert toml_file['debugging:logging'] == True
+        assert toml_file['debugging:logdir'] == '/usr/local/var/run/clu'
+        assert toml_file['userinfo:user'] == 'fish'
+        assert toml_file['userinfo:email'] == 'fish2000@gmail.com'
+        assert toml_file['userinfo:fullname'] == 'Alexander Böhn'
+        assert toml_file['userinfo:organization'] == 'Objects In Space And Time, LLC'
+        
+        flat = toml_file.flatten()
+        
+        assert flat.namespaces() == ('debugging', 'userinfo')
+        assert flat['project'] == 'clu'
+        assert flat['description'] is not None
+        assert flat['description_content_type'] == 'text/markdown'
+        assert flat['debugging:debug'] == True
+        assert flat['debugging:logging'] == True
+        assert flat['debugging:logdir'] == '/usr/local/var/run/clu'
+        assert flat['userinfo:user'] == 'fish'
+        assert flat['userinfo:email'] == 'fish2000@gmail.com'
+        assert flat['userinfo:fullname'] == 'Alexander Böhn'
+        assert flat['userinfo:organization'] == 'Objects In Space And Time, LLC'
+        
+        assert toml_file == flat
+    
     def test_toml_and_file_search(self, dirname, environment):
         # N.B. we use the “environment” fixture here to winnow out
         # any XDG variable definitions, some of which are inspected by
@@ -225,6 +265,7 @@ class TestConfig(object):
         assert TomlFile.appname in TomlFile.filename
         assert TomlFile.filename in cfgs
         
+        # Instantiate a TomlFile by searching:
         toml_file = TomlFile(extra_user_dirs=tuplize(cfgs))
         
         assert os.path.exists(toml_file.filepath)
@@ -259,12 +300,14 @@ class TestConfig(object):
         
         assert toml_file == flat
         
+        # Call “find_file(…)” directly, returning a file path:
         default_toml = TomlFile.find_file(extra_user_dirs=tuplize(cfgs))
         different_toml = TomlFile.find_file(filename='yodogg.toml', extra_user_dirs=tuplize(cfgs))
         
         assert default_toml.endswith('tests/data/config/clu-config.toml')
         assert different_toml.endswith('tests/data/config/yodogg.toml')
         
+        # Try to find a nonexistant file:
         with pytest.raises(FileNotFoundError) as exc:
             TomlFile.find_file(filename='NO-DOGG.toml', extra_user_dirs=tuplize(cfgs))
         assert "config file NO-DOGG.toml" in str(exc.value)
