@@ -10,6 +10,35 @@ class TestNaming(object):
     
     """ Run the tests for the clu.naming module. """
     
+    def test_module_inspectors_0(self):
+        from clu.naming import isbuiltin
+        
+        assert isbuiltin(callable)
+        assert isbuiltin(set)
+        assert not isbuiltin(isbuiltin)
+    
+    def test_module_inspectors_1(self):
+        from clu.naming import isnativemodule
+        
+        imaging = pytest.importorskip('PIL._imaging')
+        zict = pytest.importorskip('zict')
+        import os
+        
+        assert isnativemodule(imaging)
+        assert not isnativemodule(zict)
+        assert not isnativemodule(os) # builtins don’t qualify
+    
+    def test_module_inspectors_2(self):
+        from clu.naming import isnative
+        
+        imaging = pytest.importorskip('PIL._imaging')
+        Image = pytest.importorskip('PIL.Image')
+        
+        assert isnative(imaging.alpha_composite)
+        assert not isnative(Image)
+        assert not isnative(callable) # builtins don’t qualify
+        assert not isnative(dict) # I just said they don’t qualify
+    
     def test_dotpath_to_prefix_and_path_to_prefix(self):
         from clu.naming import dotpath_to_prefix, path_to_prefix
         
@@ -53,7 +82,7 @@ class TestNaming(object):
     
     @pytest.mark.skipif(consts.PYPY, reason="Failure on PyPy")
     @pytest.mark.nondeterministic
-    def test_qualified_name_constants(self):
+    def _test_qualified_name_constants(self):
         """ » Checking “qualified_name(¬) on items from clu.constants …” """
         from clu.constants.consts import (BASEPATH, HOSTNAME, VERBOTEN, SCRIPT_PATH, TEST_PATH)
         from clu.naming import qualified_name
@@ -125,8 +154,8 @@ class TestNaming(object):
         from clu.naming import qualified_import, qualified_name
         # is_python2_dead     = qualified_import('clu.repl.banners.is_python2_dead')
         
-        print_python_banner = qualified_import('clu.repl.print_python_banner')
-        print_warning       = qualified_import('clu.repl.print_warning')
+        print_python_banner = qualified_import('clu.repl.banners.print_python_banner')
+        print_warning       = qualified_import('clu.repl.banners.print_warning')
         Text                = qualified_import('clu.repl.ansi.Text')
         Background          = qualified_import('clu.repl.ansi.Background')
         Weight              = qualified_import('clu.repl.ansi.Weight')
